/**
  *  \file client/tiles/baseoverview.cc
  *
  *  (c) 2001-2010 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/tiles/baseoverview.h"
#include "ui/uiglobal.h"
#include "game/planet.h"
#include "game/univ.h"
#include "util/nls.h"
#include "cpluslib/format.h"
#include "game/global.h"
#include "game/ship.h"

WBaseOverviewTile::WBaseOverviewTile(GObjectType& type)
    : WObjectTypeObserverTile(font_sizes[FONT_NORMAL].scaledBy(30, 10), type)
{ }

void
WBaseOverviewTile::drawData(GfxCanvas& can)
{
    GPlanet* p = dynamic_cast<GPlanet*>(getObject());
    if (!p)
        return;

    GPlanet& pp = *p;

    GfxContext ctx;
    ctx.useCanvas(can);
    ctx.useColorScheme(getSkin());
    ctx.useFont(*fonts[FONT_NORMAL]);
    ctx.setColor(UIColor::tc_Static);
    ctx.setSolidBackground();

    int y = 0, x = 0;

    outTextF(ctx, x, y, extent.w, format(_("%d mc; %d kt supplies"), pp.getCargo(el_Money), pp.getCargo(el_Supplies)));
    y += font_heights[FONT_NORMAL];

    /* minerals */
    outTextF(ctx, x, y, 70, _("Minerals:"));
    ctx.setTextAlign(2, 0);
    outTextF(ctx, x+130, y, 60, format("%dN", pp.getCargo(el_Neutronium)));
    outTextF(ctx, x+210, y, 80, format("%dT", pp.getCargo(el_Tritanium)));
    outTextF(ctx, x+290, y, 80, format("%dD", pp.getCargo(el_Duranium)));
    outTextF(ctx, x+370, y, 80, format("%dM", pp.getCargo(el_Molybdenum)));
    y += font_heights[FONT_NORMAL];

    /* tech */
    ctx.setTextAlign(0, 0);
    outTextF(ctx, x, y, 100, _("Tech Levels:"));
    ctx.setTextAlign(2, 0);
    outTextF(ctx, x+130, y, 30, format("%dH", pp.getBaseTech(tl_Hull)));
    outTextF(ctx, x+210, y, 80, format("%dE", pp.getBaseTech(tl_Engine)));
    outTextF(ctx, x+290, y, 80, format("%dB", pp.getBaseTech(tl_Beam)));
    outTextF(ctx, x+370, y, 80, format("%dT", pp.getBaseTech(tl_Torpedo)));
    y += font_heights[FONT_NORMAL]+2;

    /* are we building something? */
    // FIXME: use getType()'s universe
    ctx.setTextAlign(0, 0);
    if (int sid = pp.getShipCloningHere(getDisplayedTurn().getCurrentUniverse())) {
        outTextF(ctx, x, y, extent.w, format(_("Cloning %s"),
                                                 getDisplayedTurn().getCurrentUniverse().getShip(sid).getName()));
    } else if (int hid = pp.getBaseBuildHull()) {
        outTextF(ctx, x, y, extent.w, format(_("Building a %s"), getHull(hid).getName()));
    } else {
        outTextF(ctx, x, y, extent.w, string_t());
    }
    y += font_heights[FONT_NORMAL];

    /* repairing something? */
    if (pp.getBaseShipyardOrder()) {
        outTextF(ctx, x, y, extent.w,
                 format(pp.getBaseShipyardOrder() == 1 ? _("Repairing %s") : _("Recycling %s"),
                        getDisplayedTurn().getCurrentUniverse().getShip(pp.getBaseShipyardId()).getFullName()));
    } else {
        outTextF(ctx, x, y, extent.w, string_t());
    }
    y += font_heights[FONT_NORMAL]+2;

    /* administrivia */
    outTextF(ctx, x, y, extent.w, format(_("Planetary FCode: %s"), pp.getFCode()));
    y += font_heights[FONT_NORMAL];
    outTextF(ctx, x, y, extent.w,
                 format(_("SB Defense: %d; Fighters: %d; Damage: %d%%"),
                        pp.getStructures(str_BaseDefense), pp.getCargo(el_Fighters), pp.getBaseDamage()));
    y += font_heights[FONT_NORMAL];
}
