/**
  *  \file game/sel.h
  *
  *  (c) 2001-2010 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_SEL_H
#define PCC_V2_GAME_SEL_H

#include "game/struct.h"
#include "cpluslib/types.h"
#include "cpluslib/string.h"
#include "cpluslib/sigslot.h"

class GUniverse;

class GSelection {
 public:
    enum {
        NUM_BITS  = NUM_SHIPS + NUM_PLANETS,
        NUM_WORDS = (NUM_BITS+31)/32,

        FIRST_SHIP_INDEX   = 0,
        FIRST_PLANET_INDEX = NUM_SHIPS
    };

    GSelection();

    void      clear();
    void      copyFrom(const GUniverse& univ);
    void      copyTo(GUniverse& univ);
    void      limitToExistingObjects(const GUniverse& univ);

    // Counts
    int       getNumShips() const;
    int       getNumPlanets() const;

    // Bitwise access
    bool      get(uint32_t index) const;
    void      set(uint32_t index, bool value);

    // Word-wise access
    uint32_t  operator[](uint32_t word_index) const;
    uint32_t& operator[](uint32_t word_index);
    static uint32_t getShipMask(uint32_t word_index);
    static uint32_t getPlanetMask(uint32_t word_index);

 private:
    uint32_t data[NUM_WORDS];
};

/* Multi-selection: */
class GMultiSelection {
 public:
    enum {
        NUM_SELECTION_LAYERS = 8
    };

    enum {
        opAnd        = '&',
        opOr         = '|',
        opXor        = '^',
        opNot        = '!',
        opCurrent    = 'c',
        opShip       = 's',
        opPlanet     = 'p',
        opFirstLayer = 'A',
        opZero       = '0',
        opOne        = '1'
    };

    static int         getCurrentSelectionLayer();
    static void        setCurrentSelectionLayer(int layer);
    static void        executeSelectionExpression(int target_layer, string_t compiled_expression);
    static GSelection& getSelectionLayer(int layer);
    static bool        isValidSelectionLayer(int layer);

    static Signal0     sig_selection_changed;
};

class ModuleInitializer;
extern const ModuleInitializer mod_sel[];

#endif
