/**
  *  \file int/binary.h
  *  \brief Interpreter: Binary Operations
  *
  *  (c) 2001-2010 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_INT_BINARY_H
#define PCC_V2_INT_BINARY_H

#include "cpluslib/types.h"

class IntValue;

/** Minor opcode for binary operations. These are stored in the "minor" field of an
    IntOpcode whose major opcode is IntOpcode::maBinary. */
enum IntBinaryOperation {
    biAnd,                      ///< Logical And.
    biOr,                       ///< Logical Or.
    biXor,                      ///< Logical Xor.
    biAdd,                      ///< Add.
    biSub,                      ///< Subtract.
    biMult,                     ///< Multiply.
    biDivide,                   ///< Divide.
    biIntegerDivide,            ///< Integer divide.
    biRemainder,                ///< Compute remainder.
    biPow,                      ///< Power.
    biConcat,                   ///< String concat, Empty annihilates result ('#').
    biConcatEmpty,              ///< String concat, Empty interpolates to empty string ('&').
    biCompareEQ,                ///< Comparison.
    biCompareEQ_NC,             ///< Comparison, ignoring case.
    biCompareNE,                ///< Comparison.
    biCompareNE_NC,             ///< Comparison, ignoring case.
    biCompareLE,                ///< Comparison.
    biCompareLE_NC,             ///< Comparison, ignoring case.
    biCompareLT,                ///< Comparison.
    biCompareLT_NC,             ///< Comparison, ignoring case.
    biCompareGE,                ///< Comparison.
    biCompareGE_NC,             ///< Comparison, ignoring case.
    biCompareGT,                ///< Comparison.
    biCompareGT_NC,             ///< Comparison, ignoring case.
    biMin,                      ///< Return minimum.
    biMin_NC,                   ///< Return minimum, ignoring case.
    biMax,                      ///< Return maximum.
    biMax_NC,                   ///< Return maximum, ignoring case.
    biFirstStr,                 ///< First part of string.
    biFirstStr_NC,              ///< First part of string, ignoring case.
    biRestStr,                  ///< Rest of string.
    biRestStr_NC,               ///< Rest of string, ignoring case.
    biFindStr,                  ///< Locate string.
    biFindStr_NC,               ///< Locate string, ignoring case.
    biBitAnd,                   ///< Bitwise and.
    biBitOr,                    ///< Bitwise or.
    biBitXor,                   ///< Bitwise xor.
    biStr,                      ///< Stringify with explicit width given.
    biATan,                     ///< Arctangent.
    biLCut,                     ///< Left cut. Remove everything before string position N.
    biRCut,                     ///< Right cut. Remove everything after string position N.
    biEndCut,                   ///< End cut. Remove everything but last N characters.
    biStrMult,                  ///< String multiply. First arg is count.
    biKeyAddParent,             ///< Add parent to keymap.
    biKeyFind                   ///< Look up key in keymap.
};

const char* getBinaryName(uint8_t op);

IntValue* executeBinaryOperation(uint8_t op, IntValue* a, IntValue* b);

#endif
