/**
  *  \file int/names.h
  *  \brief Interpreter: Variable Name Mapping
  *
  *  (c) 2001-2010 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_INT_NAMES_H
#define PCC_V2_INT_NAMES_H

#include <vector>
#include "cpluslib/types.h"
#include "cpluslib/string.h"

class Stream;

/** Variable Name Mapping. This provides a mapping from variable names to
    addresses (IntDataSegment indices) and back.

    \todo This is a really simple implementation. It is implemented the same
    way as the PCC 1.x version. A (possibly faster) version would use a
    std::map for the name->address mapping. 20100411: Map currently gives only
    a marginal speedup. */
class IntVariableNames {
    IntVariableNames& operator=(const IntVariableNames&);

    std::vector<string_t>  names;
 public:
    typedef int32 index_t;

    static const index_t nil = -1;

    // default IntVariableNames(const IntVariableNames&);
    IntVariableNames();
    ~IntVariableNames();

    index_t  lookup(string_t name) const;
    index_t  add(string_t name);
    index_t  addMaybe(string_t name);
    string_t lookupAddress(index_t index) const;
    index_t  getNumNames() const;

    void     save(Stream& s, uint32 count) const;
    void     load(Stream& s, uint32 count);
};

#endif
